## INTRODUCTION
The Reporter module is a SQL Report Generator.
The primary use case for this module is:
- Viewing Simple Statistical Distribution Reports for the web site.
- It provides some generic reports for any Drupal Site.
- Easily creating standalone Summary Reports from SQL.
- Creating Detail Reports that are linked to Summary Reports.

## FEATURES
- Renders your SQL into tables without custom coding for displaying the SQL results.
- Invisible Columns  Extra columns that can be incorporated into Links to other pages.
- External links  include the data from Invisible Columns in the URL path or Query string.
- Detail Reports - Incorporate the Query String from Summary Reports in their SQL Query.
- Parameter Tokens  in the URL can be replaced in the SQL of Detail or LookupReports.

## REQUIREMENTS
A Drupal version 10 or above Web Site

## INSTALLATION
Install as you would normally install a contributed Drupal module.
See: https://www.drupal.org/node/895232 for further information.

## USE
Go to the main Reporter Menu page to view your reports or the included generic reports.
__/admin/reports/reporter/report__

## CONFIGURATION
- Go to the configuration page __/admin/config/system/reporter-settings__
- Click the Add another report button.
- Enter the Name, Description and SQL for the report and click Save configuration.
- Go to the Report page __/admin/reports/reporter/report__ to see a list of all reports.
- Click on the link for the report with the name you just created.

## SUMMARY REPORTS
Summary Reports display the results of the supplied SQL Query as Rows of Columns.
Summary Reports are standalone pages that dont require any additional information in order to be displayed.
Reporter reports can include Invisible Columns. Invisible Columns are columns that are selected to be used as parameters in Detail Reports. Parameter Placeholder Tokens get inserted into the SQL Queries of other reports. Query parameters in the URL replace the Parameter Placeholder Tokens in the SQL of Reports when they are displayeds.

Summary Report Example SQL:
__SELECT COUNT(*) AS \`Quantity\`, \`location\` AS \`Page URL\`
FROM \`watchdog\` 
WHERE \`type\` = 'page not found' 
GROUP BY \`location\` 
ORDER BY COUNT(*) DESC__
A two column Summary Report that counts and groups pages that could not be found together.

## DETAIL REPORTS
Detail Reports display the results of the supplied SQL Query as Rows of Columns.

Detail Reports are not standalone pages they require additional information in order to be displayed. The information is supplied by adding query string arguments to the url that displays the report.

__Example:__
This url template is specifying the customer_detail report and is associating the id argument with the CUSTID Placeholder Token. Replacement tokens are all caps for clarity.
__/admin/reports/reporter/report/?report_machine_name=customer_detail&id=CUSTID__

So when an actual request comes in with a url like this:
__/admin/reports/reporter/report/?report_machine_name=customer_detail&id=1234__
The CUSTID embedded in the SQL of the customer_detail will be replaced with 1234.

SQL Template from the reports Configuration Page
__SELECT name, address, phone FROM customers_table WHERE id=CUSTID__

SQL that gets sent to the database after the Placeholder Token is replaced with real data :
__SELECT name, address, phone FROM customers_table WHERE id=1234__

The Placeholder Tokens are located in two places.
In the URL template of the previous __calling report__. The data comes from hidden columns.
In the SQL Template of the reports Configuration Page.

__Invisible Columns__
Detail reports can also include Invisible Columns. Invisible Columns are columns that are selected to be used as parameters in linking to other Detail Reports. You insert Parameter Placeholder Tokens into the SQL Queries Templates of the other reports. Query parameters in the URL replace the Parameter Placeholder Tokens in the SQL Templates of other reports. 

Since Detail Reports are not stand alone and need additional arguments to be displayed you can exclude them from being listed on the Reporter Menu Page:
__/admin/reports/reporter/report__
This is accomplished by setting the Exclude this report from the Menu Page report option.

## LOOKUP REPORTS
Lookup Reports are standalone pages that have a built in Filter Form to be displayed at the top of the report to narrow the results of the report. 

__Lookup Forms__
You can add an HTML Form to the reports Description Field. Click the Source Icon in the Description field and paste the html of the form into the existing HTML.

__A Search Form example:__

    <form action="">
       <input id="reporter_machine_name"
                    name="reporter_machine_name"
                    type="hidden"
                    value="name_of_report_to_link_to" />
       `<label for="email">Contact Email:</label><br />
       <input id="EMAIL" name="EMAIL" type="text" /><br />
       <input type="submit" value="Submit" />&nbsp;</form> 
    </form>

A form with a Name field and a Submit button.
This form links to a report named name_of_report_to_link_to.
It has an Input field for capturing data for a token named EMAIL.
That token named EMAIL can be used in the SQL of the linked report.
Placing a form like this in the Description of a report will add a Query argument to the URL.
Placing the EMAIL token in the SQL will put the value  from the URL EMAIL token in the SQL.

Search Form SQL template example
__SELECT \`mail\` AS \`Mail\`, \`name\` AS \`Name\`, \`uid\` AS \`UID\` FROM \`users_field_data\` WHERE \`mail\` LIKE '%EMAIL%'__

The SQL includes the EMAIL token. If the url of the report had query string like this:
__?reporter_machine_name=name_of_report_to_link_to&EMAIL=fred@gmail.com__
The report name is 'name_of_report_to_link_to'. The SQL would have the EMAIL Token replaced with fred@gmail.com 
and would look like this before getting submitted to the database:
__SELECT \`mail\` AS \`Mail\`, \`name\` AS \`Name\`, \`uid\` AS \`UID\` FROM \`users_field_data\` WHERE \`mail\` LIKE '%fred@gmail.com%'__
The EMAIL token would be replaced by fred@gmail.com the value entered into the form.

## OTHER LINKED PAGES
A report can specify two types of links that can optionally be placed in one of the visible columns of your reports.
__I.__ For linking to other reporter Detail Reports. The Invisible Column Names and values get added to the reports URL Query String.

You would choose this url option
    (*) Add Invisible Column Names and Values To URL Query String. (Default)

Example URL Template
__/admin/reports/reporter/report/?report_machine_name=customer_detail__

On a report named customer_detail one or more Query String arguments would be added to the URL using the Names and Values of Invisible columns.
__?invisible_column_name=invisible_column_value__

__II.__ For linking to other types of pages that don't use Query String arguments like Drupal. The Invisible Column Values get substituted for the Column Name in the reports URL template.
You would choose this url option:
     ( )  Substitute Invisible Column Values For Names In The Link URL.
Example URL Template
__/user/UID/edit__
A Placeholder Token named UID would be replaced by a Hidden Column named UID.

## CONFIGURATION OPTIONS
To configure Reporter Reports go to this page:
__/admin/config/system/reporter-settings__
The following configuration settings can be set for each report

__Name:__
The name of the report

__Machine Name:__
Use lowercase letters or numbers or an underscore. This is the name of the report used in urls. If you leave this blank one will be created for you.

__Description:__
A description of the report. Can also contain an HTML Form for filtering reports.

__SQL Template:__
The SQL SELECT Query or Template to produce the desired report. Other types of SQL that can modify the database are dis-allowed.

__Quantity of Visible Columns:__
Limit the number of Visible columns. Invisible columns appear as arguments in the query string or path of the URL of the Link column. They are used for linking to Detail Pages (optional).

__Link Column: (optional)__
Used to select which column will have an optional link. To use this, you need to SELECT the columns you want to be included in the query string of the link in your SQL.

__Link URL Template: (optional)__
This is the URL Template for the Link Column /admin/reports/reporter/report is the default path for Reporter Reports. The URL for each row is different so it can include the needed parameters for the linked page.

Reporter can customize each url by adding the parameters required for the page you are linking to.
The data for the parameters comes from Invisible Columns. Reporter supports two styles of adding parameters. 

*Reporter Report Pages*
For Reporter Report Pages the URL Query String can have the Invisible Column data added as additional Query String arguments.

If this were your URL template for the customer_detail report:
__/admin/reports/reporter/report/?report_machine_name=customer_detail__
And you had added a Hidden column to your SQL SELECT statement like this:
__customer_id AS \`CUSTID\`,__
Then Reporter would make the URL for the link and replace the Placeholder Token:
__/admin/reports/reporter/report/?report_machine_name=customer_detail&CUSTID=1234__
CUSTID being the Name of the Hidden Column. And 1234 being the Value of the Hidden Column.

*Other types of pages*
For other kinds of non reporter pages that dont use URL Query Strings like Drupal Pages Reporter can substitute Placeholder Tokens in the URL path.

If this were the path to a Drupal Users edit page for user id 1234:
__/user/1234/edit__

You could specify a URL Template like this:
__/user/UID/edit__

And you had added a Hidden column to your SQL SELECT statement like this:
__uid AS \`UID\`,__
Then Reporter would make the URL for the link and replace the Placeholder Token:
__/user/1234/edit__

__Link Style: (optional)__
You can specify the style URL to use in the Reports configuration.
How To Add Invisible Column Data To The Link URL:
    (*) Add Invisible Column Names and Values To URL Query String. (Default)
    ( )  Substitute Invisible Column Values For Names In The Link URL.


__Exclude this report from the Menu Page: (optional)__
Exclude this report from the Menu Page. This would be done for Detail pages that need query arguments and cannot return results without the query argument.

## APPENDIX
__SUGGESTION__
You should refer to your Placeholder Tokens with all uppercase letters and numbers.

This is not a technical requirement but this convention makes it easier to recognize the difference between SQL Table and Column names and Placeholder Tokens in your SQL and URLS.

__DEBUGGING__
To make it easier to determine what the SQL looks like after the Placeholder Tokens have been replaced by the values from the URL Query String. Add the following Query String Fragment to the URL:
__&debug=true__

It will cause the SQL that is being sent to the database to be displayed in a green box at the top of the report page.

__SQL SAFETY__
Reporter is only useful for retrieving data from the database. SQL that INSERTS or UPDATES data cannot be saved or retrieved by Reporter.

## MAINTAINERS
Current maintainers for Drupal 10:
- Seth Snyder - https://www.drupal.org/u/seth-snyder
